/*------------------------------------
Delete Double INSERT with TRUNCATE
------------------------------------*/
USE DMLTest;

-- Create a table with a check constraint, then INSERT 1,000,000 records
CREATE TABLE DoubleInsert (Id int IDENTITY (1,1) PRIMARY KEY CLUSTERED, SomeValue int, SomeText varchar(100));

ALTER TABLE DoubleInsert ADD CONSTRAINT ck_SomeValue CHECK (SomeValue > 0 AND SomeValue < 9999999);

INSERT DoubleInsert WITH (TABLOCK) (SomeValue, SomeText)
SELECT (1000001-Id), REPLICATE('ABCDE',20)
FROM OneMillionRecords;

SET STATISTICS IO ON;

-- DELETE 90% of the table
DELETE
FROM DoubleInsert
WHERE SomeValue > 100000 
-- How many logical reads?








-- Let's reset the table to 1,000,000 records
TRUNCATE TABLE DoubleInsert

INSERT DoubleInsert WITH (TABLOCK) (SomeValue, SomeText)
SELECT (1000001-Id), REPLICATE('ABCDE',20)
FROM OneMillionRecords;


-- Let's try creating a new table, INSERT only what we want to keep, then rename the tables
CREATE TABLE DoubleInsertNew (Id int PRIMARY KEY CLUSTERED, SomeValue int, SomeText varchar(200));

INSERT DoubleInsertNew WITH (TABLOCK) (Id, SomeValue, SomeText)
SELECT Id, SomeValue, SomeText
FROM DoubleInsert
WHERE SomeValue <= 100000
-- You can probably guess this has less logical reads

-- Now we TRUNCATE the original table and add back what we need
TRUNCATE TABLE DoubleInsert;

ALTER TABLE DoubleInsert NOCHECK CONSTRAINT ck_SomeValue;

SET IDENTITY_INSERT DoubleInsert ON;

INSERT DoubleInsert WITH (TABLOCK) (Id, SomeValue, SomeText)
SELECT Id, SomeValue, SomeText
FROM DoubleInsertNew;
-- Look ma, less logical reads!

SET IDENTITY_INSERT DoubleInsert OFF;

ALTER TABLE DoubleInsert CHECK CONSTRAINT ck_SomeValue;

